﻿using System.Collections.ObjectModel;
using AsReaderP252BAndroidDemo.Platforms.Android;
using Com.Asreader.P252b.Barcode;
using Com.Asreader.P252b.Device.Type;
using Com.Asreader.P252b.Rfid.Type;
using Java.Nio.Charset;

namespace AsReaderP252BAndroidDemo;

public partial class BarcodeScanPage : ContentPage
{

    private AsReaderP252BMAUIManager asReaderP252BMAUIManager;
    private AsReaderP252BDelegate aAsReaderP252BDelegate;
    private ObservableCollection<ItemData> BarcodeItems = new ObservableCollection<ItemData>();
    public ObservableCollection<ItemData> ItemData { get { return BarcodeItems; } }
    private Dictionary<string, ItemData> dataDic = new Dictionary<string, ItemData>();

    private AsReaderP252BActionState actionState;
    int count;
    int currentEncode = 0;
    public BarcodeScanPage()
	{
        InitializeComponent();
        actionState = AsReaderP252BActionState.Stop;
        asReaderP252BMAUIManager = AsReaderP252BMAUIManager.sharedInstance();
        aAsReaderP252BDelegate = AsReaderP252BDelegate.sharedInstance();
    }
    protected override void OnAppearing()
    {
        base.OnAppearing();
        BarcodeListView.ItemsSource = BarcodeItems;
        aAsReaderP252BDelegate.OnStateChanged = (AsReaderP252BConnectionState state) =>
        {
            OnStateChanged(state);
        };
        aAsReaderP252BDelegate.OnReadBarcode = (AsReaderP252BBarcodeType p1, byte[] p2) =>
        {
            String barcode = "";
            switch (currentEncode)
            {
                case 0:
                    {
                        barcode = System.Text.Encoding.ASCII.GetString(p2);
                    }
                    break;
                case 1:
                    {
                        barcode = System.Text.Encoding.UTF8.GetString(p2);

                    }
                    break;
                case 2:
                    {
                        Charset mCharset_ShiftJIS = Charset.ForName("Shift_JIS");
                        Java.Lang.String data = new Java.Lang.String(p2, mCharset_ShiftJIS);
                        barcode = data.ToString();
                    }
                    break;
            }
            string barcodeType = p1.ToString();
            OnReadBarcode(barcodeType, barcode);
        };
        aAsReaderP252BDelegate.OnKeyEvent = (AsReaderP252BKeyState p1) =>
        {
            return OnKeyEvent(p1);
        }; 
        aAsReaderP252BDelegate.OnActionChanged = (AsReaderP252BActionState p1) =>
        {
            OnActionChanged(p1);
        }; 
    }
    protected override void OnDisappearing()
    {
        base.OnDisappearing();
        asReaderP252BMAUIManager.StopDecode();
    }
    void StartScanButton_Clicked(object sender, EventArgs e)
    {
        if (actionState == AsReaderP252BActionState.StartDecode)
        {
            asReaderP252BMAUIManager.StopDecode();
        }
        else if (actionState == AsReaderP252BActionState.Stop)
        {
            asReaderP252BMAUIManager.StartDecode();
        }
    }
    void ClearScanButton_Clicked(object sender, EventArgs e)
    {
        count = 0;
        CountLabel.Text = count.ToString();
        dataDic.Clear();
        BarcodeItems.Clear();
    }
    void ASCIIButton_Clicked(object sender, EventArgs e)
    {
        ASCIIButton.TextColor = Colors.Black;
        UTF8Button.TextColor = Colors.Gray;
        ShiftJISButton.TextColor = Colors.Gray;
        currentEncode = 0;
    }
    void UTF8Button_Clicked(object sender, EventArgs e)
    {
        ASCIIButton.TextColor = Colors.Gray;
        UTF8Button.TextColor = Colors.Black;
        ShiftJISButton.TextColor = Colors.Gray;
        currentEncode = 1;
    }
    void ShiftJISButton_Clicked(object sender, EventArgs e)
    {
        ASCIIButton.TextColor = Colors.Gray;
        UTF8Button.TextColor = Colors.Gray;
        ShiftJISButton.TextColor = Colors.Black;
        currentEncode = 2;
    }
    private void OnStateChanged(AsReaderP252BConnectionState state)
    {
        if (state != AsReaderP252BConnectionState.Connected)
        {
            MainThread.BeginInvokeOnMainThread(() =>
            {
                Navigation.PopAsync();
            });
        }
    }
    private void OnReadBarcode(string p1,  string p3)
    {
        ItemData InventoryDataChanged;
        if (dataDic.ContainsKey(p1 + p3))
        {
            InventoryDataChanged = dataDic[p1 + p3];
            InventoryDataChanged.Count++;
            dataDic[p1 + p3] = InventoryDataChanged;
        }
        else
        {
            int nowCount = dataDic.Count;
            InventoryDataChanged = new ItemData(p1, p3, 1);
            dataDic.Add(p1 + p3, InventoryDataChanged);
            BarcodeItems.Add(InventoryDataChanged);
        }
        count++;
        MainThread.BeginInvokeOnMainThread(() =>
        {
            BarcodeListView.ItemsSource = null;
            BarcodeListView.ItemsSource = BarcodeItems;
            CountLabel.Text = count.ToString();
        });
    }
    private bool OnKeyEvent(AsReaderP252BKeyState p1)
    {
        if (p1 == AsReaderP252BKeyState.KeyDown)
        {
            if (actionState == AsReaderP252BActionState.StartDecode)
            {
                asReaderP252BMAUIManager.Stop();
            }
            else
            {
                asReaderP252BMAUIManager.StartDecode();
            }
        }
        return false;
    } 
    private void OnActionChanged(AsReaderP252BActionState p1)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            if (p1 == AsReaderP252BActionState.StartDecode)
            {
                actionState = AsReaderP252BActionState.StartDecode;
                StartScanButton.Text = "Stop Scan";
            }
            else if (p1 == AsReaderP252BActionState.Stop)
            {
                actionState = AsReaderP252BActionState.Stop;
                StartScanButton.Text = "Start Scan";
            }
        });

    }
}
