﻿using Android.Bluetooth;
using AsReaderP252BAndroidDemo.Platforms.Android;
using Com.Asreader.P252b.Device.Type;

namespace AsReaderP252BAndroidDemo;

public partial class MainPage : ContentPage
{
    private AsReaderP252BMAUIManager asReaderP252BMAUIManager;
    private AsReaderP252BDelegate asReaderP252BDelegate;
    private string appVersion;
    private string dllVereison;
    private string sdkVersion;
    private string firmwareVersion;
    private string rfModuleVersion;
    private string serialVersion;
    private string globalBandVersion;
    private int battery;
    private List<BluetoothDevice> arrDeviceList;

    public MainPage()
	{
		InitializeComponent();
        ConnectButton.Text = "Connect";
        BluetoothSearchButton.Text = "Search";
        MainActivityIndicator.IsRunning = false;
        arrDeviceList = new List<BluetoothDevice>();
        asReaderP252BMAUIManager = AsReaderP252BMAUIManager.sharedInstance();
        asReaderP252BDelegate = AsReaderP252BDelegate.sharedInstance();
    }
    protected override void OnAppearing()
    {
        base.OnAppearing();
        if (asReaderP252BMAUIManager.isConnect == true)
        {
            reloadUI(true);
        }
        else
        {
            reloadUI(false);
        }
        asReaderP252BDelegate.OnStateChanged = (AsReaderP252BConnectionState state) =>
        {
            OnStateChanged(state);
        };
        asReaderP252BDelegate.OnKeyEvent = (AsReaderP252BKeyState p1) =>
        {
            return false;
        };
        asReaderP252BDelegate.OnFoundDeviceFinished = () =>
        {
            OnFoundDeviceFinished();
        };
        asReaderP252BDelegate.OnReceivedDevice = (BluetoothDevice p0) =>
        {
            OnReceivedDevice(p0);
        };
    }
    private void OnStateChanged(AsReaderP252BConnectionState state)
    {
        if (state == AsReaderP252BConnectionState.Connected)
        {
            reloadUI(true);
        }
        else if (state == AsReaderP252BConnectionState.Connecting)
        {
            StartActivityIndicator();
        }
        else if (state == AsReaderP252BConnectionState.Disconnected)
        {
            reloadUI(false);
        } 
    }
    private void OnFoundDeviceFinished()
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            asReaderP252BMAUIManager.StopDiscovery();
            if (arrDeviceList.Count > 0)
            {
                showAlert(title: "Search for bluetooth", message: "", bleDevices: arrDeviceList);
            } else
            {
                DisplayAlert("No reader found.", "Check the power of the reader.", "OK");
            }
            StopActivityIndicator();
        });
    }
    private void OnReceivedDevice(BluetoothDevice device)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            if (!arrDeviceList.Contains(device))
            {
                if (device.Name != null && device.Name.Contains("PADDLE-Type"))
                {
                    arrDeviceList.Add(device);
                }
            }
        });
    }
    private void reloadUI(bool status)
    {
        if (status == true)
        {
            Task.Run(() =>
            {
                appVersion = asReaderP252BMAUIManager.GetAPPVersion();
                dllVereison = asReaderP252BMAUIManager.GetAsReader251GSDKDLL();
                sdkVersion = asReaderP252BMAUIManager.GetSDKVersion();
                firmwareVersion = asReaderP252BMAUIManager.GetFirmwareVersion();
                rfModuleVersion = asReaderP252BMAUIManager.GetRFModuleVersion();
                serialVersion = asReaderP252BMAUIManager.GetSerialNumber();
                battery = asReaderP252BMAUIManager.GetBatteryStatus();
                globalBandVersion = asReaderP252BMAUIManager.GetRegion();
                MainThread.BeginInvokeOnMainThread(() =>
                {
                    ConnectButton.Text = "Disconnect";
                    APPVersionLabel.Text = appVersion;
                    DLLVersionLabel.Text = dllVereison;
                    SDKVersionLabel.Text = sdkVersion;
                    FirmwareVersionLabel.Text = firmwareVersion;
                    UHFLabel.Text = rfModuleVersion;
                    SNLabel.Text = serialVersion;
                    RegionLabel.Text = globalBandVersion;
                    CheckBattery(battery);
                    BarocdeScanLabel.TextColor = Colors.Black;
                    RFIDInventoryLabel.TextColor = Colors.Black;
                    LogoImage.Source = "asreader_log_on.png";
                    StopActivityIndicator();
                });
            });
        }
        else
        {
            MainThread.BeginInvokeOnMainThread(() =>
            {
                ConnectButton.Text = "Connect";
                APPVersionLabel.Text = "";
                DLLVersionLabel.Text = "";
                SDKVersionLabel.Text = "";
                FirmwareVersionLabel.Text = "";
                UHFLabel.Text = "";
                SNLabel.Text = "";
                RegionLabel.Text = "";
                BatteryImage.Source = "battery0.png";
                BarocdeScanLabel.TextColor = Colors.LightGray;
                RFIDInventoryLabel.TextColor = Colors.LightGray;
                LogoImage.Source = "asreader_log_off.png";
                StopActivityIndicator();
            });
        }
    }
    public void StartActivityIndicator()
    {
        MainActivityIndicator.IsRunning = true;
        ConnectButton.IsEnabled = false;
        BluetoothSearchButton.IsEnabled = false;
    }
    public void StopActivityIndicator()
    {
        MainActivityIndicator.IsRunning = false;
        ConnectButton.IsEnabled = true;
        BluetoothSearchButton.IsEnabled = true;
    }
    private void CheckBattery(int battery)
    {
        switch (battery)
        {
            case 0:
                BatteryImage.Source = "battery0.png";
                break;
            case 1:
                BatteryImage.Source = "battery1.png";
                break;
            case 2:
                BatteryImage.Source = "battery2.png";
                break;
            case 3:
                BatteryImage.Source = "battery3.png";
                break;
            case 4:
                BatteryImage.Source = "battery4.png";
                break;
            default:
                return;
        }
    }
    public void ConnectButton_Clicked(object sender, EventArgs e)
    {
        if (!asReaderP252BMAUIManager.isConnect)
        {
            StartActivityIndicator();
            asReaderP252BMAUIManager.ConnectDevice();
        }
        else
        {
            StartActivityIndicator();
            asReaderP252BMAUIManager.DisconnectDevice();
        }
    }
    [Obsolete]
    public void BluetoothSearchButton_Clicked(object sender, EventArgs e)
    {
        //Device.StartTimer(TimeSpan.FromSeconds(15.0), showSegment);
        StartActivityIndicator();
        arrDeviceList.Clear();
        asReaderP252BMAUIManager.StartDiscovery();
    }
    //private bool showSegment()
    //{
    //    MainThread.BeginInvokeOnMainThread(() =>
    //    {
    //        MainActivityIndicator.IsRunning = false;
    //        ConnectButton.IsEnabled = true;
    //        BluetoothSearchButton.IsEnabled = true;
    //    });
    //    return false;
    //}
    async void BarocdeScanClicked(object sender, EventArgs e)
    {
        if (asReaderP252BMAUIManager.isConnect)
        {
            await Navigation.PushAsync(new BarcodeScanPage());
        }
    }
    async void RFIDInventoryClicked(object sender, EventArgs e)
    {
        if (asReaderP252BMAUIManager.isConnect)
        {
            await Navigation.PushAsync(new RFIDInventoryPage());
        }
    }
    private async void showAlert(string title, string message, List<BluetoothDevice> bleDevices)
    {
        List<string> list = new List<string>();
        Dictionary<string, string> dictionary = new Dictionary<string, string>();
        for (int i = 0; i < (int)bleDevices.Count; i++)
        {
            BluetoothDevice device = bleDevices[i];
            list.Add(device.Name);
            dictionary.Add(device.Name, device.Address);
        }
        string[] buttons = list.ToArray();
        var name = await DisplayActionSheet(title, "Cancel", "", buttons);
        if (dictionary.ContainsKey(name))
        {
            string macAddress = dictionary[name];
            asReaderP252BMAUIManager.ConnectBluetoothDevice(macAddress);
        }
    }
}


